/***************************************************************************
 *   Copyright (C) 2006 by Alexios Beveratos   *
 *   alexios@beveratos.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "gpstabledata.h"
#include <qdir.h>
#include <qdatetime.h>
#include <qstring.h>
#include <math.h>

gpsTableData::gpsTableData(QWidget *parent, const char *name)
	: QTable(parent, name)
{
	setNumCols(7);
	setSelectionMode(QTable::MultiRow);

	QStringList ColHeader;
	ColHeader << "Altitude" << "Latitude" << "Longitude" << "Time" << "Speed (km/h)"<< "Image" <<"Check";
	setColumnLabels(ColHeader);
	hide = FALSE;
}


gpsTableData::~gpsTableData()
{
}

void gpsTableData::openGpsFile( const KURL& currentDir)
{
	// Search for valid .gps file and parse it 
	QDir d( currentDir.pathOrURL(), "*.gps");
	//for ( int i = 0; i < d.count(); i++ )
	if (d.count()!=0)
	{
		ParseTrackFile( (d.absPath()+"/"+d[0]));
	}
	
}

void gpsTableData::ParseTrackFile(QString trackFile)
{
	QFile *m_TrackFile = new QFile ( trackFile );
	QString TrackLine, Distance;
	QStringList TrackLinesList;
	QDateTime gpsTime;
	int row=0;
	int col=1;
	
	m_TrackFile->open(IO_ReadOnly);
	
	while ( !m_TrackFile->atEnd() ) // Read all lines and Enter them in a QStringList
	{
		m_TrackFile->readLine(TrackLine, 200);	
		if (TrackLine.section('"',1,1)=="trackpoint")
		{
			TrackLinesList <<  TrackLine;
		}
	}

	m_TrackFile->close(); // closes the file

	setNumRows( TrackLinesList.count() ); // Create the number of rows needed

	for ( QStringList::Iterator it = TrackLinesList.begin(); it != TrackLinesList.end(); ++it )
	{
		col=1;
		TrackLine = *it;
		if (TrackLine.section('"',1,1)=="trackpoint")
		{
			for (col=1;col!=5;col++)
			{	
				if (TrackLine.section('"',(2*col),(2*col)).stripWhiteSpace() == "unixtime=")
				{
					gpsTime.setTime_t( TrackLine.section('"',(2*col+1),(2*col+1)).stripWhiteSpace().toInt() );
					this->setText(row,col-1, gpsTime.toString(Qt::ISODate) );
				}else
				{
					this->setText( row,col-1,TrackLine.section('"',(2*col+1),(2*col+1)).stripWhiteSpace() );
				}
			}
		
			if (row!=0){
				this->setText(row,4, Distance.setNum( CalculateDistance(row) /CalculateTimeDiff(row)*3600. ) );
			}
			this->setItem ( row, 6, new QCheckTableItem(this, "") );
			
			row++;
		}
	}


}


double gpsTableData::CalculateDistance(int row) //double lon1,double lat1,double lon2,double lat2)
{
	double pi = 3.1415968;
	int radiusEarth = 6378;
			
	double lat1 =(this->text(row,2)).toDouble()*pi/180;
	double lat2 =(this->text(row-1,2)).toDouble()*pi/180;
	double lon1 =(this->text(row,1)).toDouble()*pi/180;
	double lon2 =(this->text(row-1,1)).toDouble()*pi/180;

	return acos(cos(lat1)*cos(lon1)*cos(lat2)*cos(lon2) + cos(lat1)*sin(lon1)*cos(lat2)*sin(lon2) + sin(lat1)*sin(lat2)) * radiusEarth;

}

int gpsTableData::CalculateTimeDiff(int row)
{
	QDateTime UnixTime = QDateTime::fromString( this->text(row-1,3), Qt::ISODate);
	return UnixTime.secsTo( QDateTime::fromString(this->text(row,3),Qt::ISODate))	;

}

void gpsTableData::hideShow()
{
	if (!hide)
	{
	for (int i=0; i<this->numRows(); i++)
		{
		if (this->text(i,5)==QString::null)
			this->hideRow(i);
		}
	}
	else{
	for (int i=0; i<this->numRows(); i++)
		{
			this->showRow(i);
		}
	}	
	hide=!hide;
}