/***************************************************************************
 *   Copyright (C) 2006 by Alexios Beveratos   *
 *   alexios@beveratos.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "kgpsimagetag.h"
#include "pref.h"
#include "exif_gps.h"

#include <qdragobject.h>
#include <kprinter.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>

#include <kglobal.h>
#include <klocale.h>
#include <kiconloader.h>
#include <kdeversion.h>
#include <kstatusbar.h>
#include <kaccel.h>
#include <kio/netaccess.h>
#include <kfiledialog.h>
#include <kconfig.h>
#include <kurl.h>
#include <kurldrag.h>
#include <kurlrequesterdlg.h>

#include <kstdaccel.h>
#include <kaction.h>
#include <kstdaction.h>

kGpsImageTag::kGpsImageTag()
    : KMainWindow( 0, "kGpsImageTag" ),
      m_view(new kGpsImageTagView(this))
{
    // accept dnd
    setAcceptDrops(true);

    // tell the KMainWindow that this is indeed the main widget
    setCentralWidget(m_view);
    
    this->setIcon( KGlobal::iconLoader()->loadIcon("kgpsimagetag", KIcon::Desktop) );

    // then, setup our actions
    setupActions();
    // and a status bar
    statusBar()->show();
    // Apply the create the main window and ask the mainwindow to
		// automatically save settings if changed: window size, toolbar
    // position, icon size, etc.  Also to add actions for the statusbar
		// toolbar, and keybindings if necessary.
    setupGUI();

    // allow the view to change the statusbar and caption
    connect(m_view, SIGNAL(signalChangeStatusbar(const QString&)),
            this,   SLOT(changeStatusbar(const QString&)));
    connect(m_view, SIGNAL(signalChangeCaption(const QString&)),
            this,   SLOT(changeCaption(const QString&)));

}

kGpsImageTag::~kGpsImageTag()
{
}

void kGpsImageTag::setupActions()
{
 //   KStdAction::preferences(this, SLOT(optionsPreferences()), actionCollection());
    KAction *CorrelateData = new KAction ( i18n("Correlate Data"), "blend", 
			    	 CTRL+Key_R , m_view, SLOT (correlateAll() ), actionCollection(),
			     	"Correlate Data");
    
    KAction *Hide_Show = new KAction ( i18n("Hide/Show"), "eraser", 
			 	CTRL+Key_E , m_view, SLOT (hideShow() ), actionCollection(),
			 	"Hide_Show");
    
    KAction *Tag_Photos = new KAction (i18n("Tag Selected Photos"), "kontact_todo",
				       CTRL+Key_T, this, SLOT (tagPhotos() ), actionCollection(),
				       "Tag_Photos");
    
    KAction *Select_All = new KAction (i18n("Select All"), "button_ok",
				       CTRL+Key_T, this, SLOT (SelectAll() ), actionCollection(),
				       "Select_All");
    
    KStdAction::up(m_view->dirBrowser(), SLOT(cdUp () ), actionCollection() );
    KStdAction::back(m_view->dirBrowser(), SLOT(back () ), actionCollection() );
    KStdAction::forward(m_view->dirBrowser(), SLOT(forward () ), actionCollection() );
    KStdAction::home(m_view->dirBrowser(), SLOT(home () ), actionCollection() );
}

void kGpsImageTag::saveProperties(KConfig *config)
{
    // the 'config' object points to the session managed
    // config file.  anything you write here will be available
    // later when this app is restored

}

void kGpsImageTag::readProperties(KConfig *config)
{
    // the 'config' object points to the session managed
    // config file.  this function is automatically called whenever
    // the app is being restored.  read in here whatever you wrote
    // in 'saveProperties'
}

void kGpsImageTag::dragEnterEvent(QDragEnterEvent *event)
{
    // accept uri drops only
    event->accept(KURLDrag::canDecode(event));
}

void kGpsImageTag::dropEvent(QDropEvent *event)
{
    // this is a very simplistic implementation of a drop event.  we
    // will only accept a dropped URL.  the Qt dnd code can do *much*
    // much more, so please read the docs there
}

void kGpsImageTag::optionsPreferences()
{
    // popup some sort of preference dialog, here
    kGpsImageTagPreferences dlg;
    if (dlg.exec())
    {
        // redo your settings
    }
}

void kGpsImageTag::changeStatusbar(const QString& text)
{
    // display the text on the statusbar
    statusBar()->message(text);
}

void kGpsImageTag::changeCaption(const QString& text)
{
    // display the text on the caption
    setCaption(text);
}


void kGpsImageTag::tagPhotos()
{
	
	/*const KFileItemList *fileList = new KFileItemList(); 
	fileList = m_view->dirBrowser()->selectedItems();
	KFileItemListIterator kit( *fileList );
	
	for ( kit ; kit.current(); ++kit )
	{
		Exif_GPS::tagPhoto( 100.0, 43.606,1.4423, kit);
	}*/
	KURL baseDir(m_view->dirBrowser()->url());
	for (int i=0;i<m_view->GpsData()->numRows ();i++)
	{
		QCheckTableItem* item = dynamic_cast<QCheckTableItem*>(m_view->GpsData()->item(i,6));
		bool Checked = item->isChecked (); //item && item->isChecked();
		if ( Checked )
		{
			QString file = baseDir.path()+m_view->GpsData()->text(i,5);
			double Alt=m_view->GpsData()->text(i,0).toDouble();
			double Lat=m_view->GpsData()->text(i,1).toDouble();
			double Lon=m_view->GpsData()->text(i,2).toDouble();
			
			Exif_GPS::tagPhoto(Alt,Lat,Lon,file);
		}
		
	}
	
	
}

void kGpsImageTag::SelectAll()
{
	for (int i=0;i<m_view->GpsData()->numRows ();i++)
	{
		QCheckTableItem* item = dynamic_cast<QCheckTableItem*>(m_view->GpsData()->item(i,6));
		if (m_view->GpsData()->text(i,5)!=QString::null)
		{
			item->setChecked(TRUE);
		}		
	}

}


#include "kgpsimagetag.moc"
