/***************************************************************************
 *   Copyright (C) 2006 by Alexios Beveratos   *
 *   alexios@beveratos.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "kgpsimagetagview.h"
#include "exif_gps.h"

#include <qpainter.h>
#include <qlayout.h>
#include <qdatetime.h>
#include <qptrlist.h>

#include <kurl.h>
#include <klibloader.h>
#include <kmessagebox.h>
#include <krun.h>
#include <klocale.h>
#include <kactioncollection.h>
#include <kmainwindow.h>
#include <kurl.h>
#include <kmessagebox.h>
#include <kprogress.h>

#include <qtoolbar.h>


kGpsImageTagView::kGpsImageTagView(QWidget *parent)
    : QWidget(parent),
      DCOPObject("kGpsImageTagIface")
{
    // setup our layout manager to automatically add our widgets
    QVBoxLayout *top_layout = new QVBoxLayout(this);
    top_layout->setAutoAdd(true);
    
    m_split = new QSplitter( this );
    m_split->setOrientation( QSplitter::Vertical);
    
    this->PrepareIconView();

    m_GpsData = new gpsTableData(m_split);

    connect(m_dirBrowser, SIGNAL(urlEntered(const KURL&)),m_GpsData,SLOT(openGpsFile(const KURL&)));
    connect(m_dirBrowser, SIGNAL(fileHighlighted (const KFileItem* )), 
	     this, SLOT( matchGPSAndPhoto(const KFileItem*) ) );
    this->keepVisible=TRUE;
}

kGpsImageTagView::~kGpsImageTagView()
{
}

KDirOperator *kGpsImageTagView::dirBrowser()
{
	return m_dirBrowser;
}

gpsTableData *kGpsImageTagView::GpsData()
{
	return m_GpsData;
}

void kGpsImageTagView::slotSetTitle(const QString& title)
{
    emit signalChangeCaption(title);
}

void kGpsImageTagView::PrepareIconView()
{
    m_dirBrowser = new KDirOperator( KURL( "file:/home/" ), m_split );
    m_dirBrowser->home();
    m_PhotoIcons = new KFileIconView(m_dirBrowser,"Icons" );
    m_PhotoIcons->setIconSize(KIcon::SizeMedium);
    m_PhotoIcons->showPreviews();
    
    m_dirBrowser->setView(m_PhotoIcons);

    (dynamic_cast<QWidget*> (m_dirBrowser))->setMinimumSize (60,150);
    
    m_split->setResizeMode(dynamic_cast<QWidget*> (m_dirBrowser),QSplitter::Auto);
}

void kGpsImageTagView::matchGPSAndPhoto(const KFileItem* selectedFile)
{
	QDateTime gpsTime;
	m_GpsData->clearSelection();
	m_GpsData->selectRow (-1);
	bool m_exactMatch=FALSE;
	
	if (!( (selectedFile->mimetype()=="image/jpeg") || (selectedFile->mimetype()=="image/tiff") ))
	{
		return;
	}

	QDateTime photoTime = Exif_GPS::extractDateTime(selectedFile);
			
	for (int i=0;i!=m_GpsData->numRows ();i++)
	{
		if ( abs (photoTime.secsTo( QDateTime::fromString(m_GpsData->text(i,3),Qt::ISODate))  ) < 30)
		{
			//m_GpsData->selectRow (i);
			m_GpsData->setText(i,5,selectedFile->name());
			m_exactMatch=TRUE;
			if (this->keepVisible)
			{
				m_GpsData->ensureCellVisible ( i,1 );
			}
		}
	}
	if (!m_exactMatch)
	{
		Extrapolate(selectedFile);
	}
}

void kGpsImageTagView::Extrapolate(const KFileItem* selectedFile)
{
	QDateTime minDate,maxDate;
	minDate.setTime_t(0);
	maxDate.setTime_t(0);
	int i;
	float ponderation, newLat, newLon, newAlt;
	QDateTime photoTime = Exif_GPS::extractDateTime(selectedFile);

	for (i=1;i!=m_GpsData->numRows ();i++)
	{
		if ( photoTime.secsTo( QDateTime::fromString(m_GpsData->text(i,3),Qt::ISODate)) > 0)
		{
			break;
		}
	}
	minDate = QDateTime::fromString(m_GpsData->text(i-1,3),Qt::ISODate);
	maxDate = QDateTime::fromString(m_GpsData->text(i,3),Qt::ISODate);
	
	ponderation = ( (float) minDate.secsTo( photoTime )) / (float) minDate.secsTo(maxDate) ;
	
	double lat1 =(m_GpsData->text(i-1,1)).toDouble();
	double lat2 =(m_GpsData->text(i,1)).toDouble();
	double lon1 =(m_GpsData->text(i-1,2)).toDouble();
	double lon2 =(m_GpsData->text(i,2)).toDouble();
	double alt1 =(m_GpsData->text(i-1,0)).toDouble();
	double alt2 =(m_GpsData->text(i,0)).toDouble();
	
	newLat= lat1+(lat2-lat1)*ponderation;
	newLon= lon1+(lon2-lon1)*ponderation;
	newAlt= alt1+(alt2-alt1)*ponderation;
	
	
	m_GpsData->insertRows ( i );
	m_GpsData->setText(i, 0, QString::number( newAlt) );
	m_GpsData->setText(i, 1, QString::number( newLat) ); 
	m_GpsData->setText(i, 2, QString::number( newLon) );
	m_GpsData->setText(i, 3, photoTime.toString(Qt::ISODate) );
	m_GpsData->setText(i, 5, selectedFile->name());
	m_GpsData->setItem(i, 6, new QCheckTableItem(m_GpsData, QString("extrapolated") ) );
	//m_GpsData->selectRow (i);
	if (this->keepVisible)
	{
		m_GpsData->ensureCellVisible ( i,1 );
	}
	
	
}

void kGpsImageTagView::correlateAll()
{
	KDirLister lister;
	lister.openURL(m_dirBrowser->url());
	KFileItemList fileList = lister.items(KDirLister::AllItems);
	
	KFileItemListIterator kit( fileList );
	this->keepVisible=FALSE;
		
	KProgressDialog *m_progress = new KProgressDialog(dynamic_cast<QWidget*>(this),"progress", "Correlating","Correlating Photons",TRUE);
	
	
	m_progress->setAutoClose(TRUE);
	m_progress->setAllowCancel(TRUE);
	m_progress->setMinimumDuration(150);
	m_progress->progressBar()->setTotalSteps(kit.count());
	//Very ugly but otherwise does not show up
	m_progress->show();
	// End ugly
	
	int i=0;
	
	for ( kit ; kit.current(); ++kit )
	{
		i++;
		matchGPSAndPhoto(kit);	
		m_progress->progressBar()->setProgress(i);
	}
	
	KMessageBox::information(0,i18n("Photos successfully correlated"));
	this->keepVisible=TRUE;

}
		
void kGpsImageTagView::hideShow()
{
	m_GpsData->hideShow();
}

#include "kgpsimagetagview.moc"
