/***************************************************************************
 *   Copyright (C) 2006 by Alexios Beveratos   *
 *   alexios@beveratos.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef _KGPSIMAGETAGVIEW_H_
#define _KGPSIMAGETAGVIEW_H_

#include <qwidget.h>
#include <kparts/part.h>
#include <kgpsimagetagiface.h>
#include <qsplitter.h>
#include <kfilepreview.h>
#include <kdiroperator.h>

#include "gpstabledata.h"


class QPainter;
class KURL;
class KDirOperator;

/**
 * This is the main view class for kGpsImageTag.  Most of the non-menu,
 * non-toolbar, and non-statusbar (e.g., non frame) GUI code should go
 * here.
 *
 * This kgpsimagetag uses an HTML component as an example.
 *
 * @short Main view
 * @author Alexios Beveratos <alexios.beveratos@lpn.cnrs.fr>
 * @version 0.1
 */
class kGpsImageTagView : public QWidget, public kGpsImageTagIface
{
    Q_OBJECT
public:
    kGpsImageTagView(QWidget *parent);
    
    virtual ~kGpsImageTagView();

     KDirOperator *dirBrowser();
     gpsTableData *GpsData();

signals:
    /**
     * Use this signal to change the content of the statusbar
     */
    void signalChangeStatusbar(const QString& text);

    /**
     * Use this signal to change the content of the caption
     */
    void signalChangeCaption(const QString& text);

    
public slots :
	void correlateAll();	
	void hideShow();
	
private slots:
    void slotSetTitle(const QString& title);
    void matchGPSAndPhoto(const KFileItem*);

private:
	KDirOperator *m_dirBrowser;
	KFileIconView *m_PhotoIcons;
	QSplitter *m_split;	
	gpsTableData *m_GpsData;
	
	void PrepareIconView();
	void Extrapolate(const KFileItem*);
	bool keepVisible;
};


#endif // _KGPSIMAGETAGVIEW_H_
